;; Generated package description from altcaps.el  -*- no-byte-compile: t -*-
(define-package "altcaps" "1.0.0" "Apply alternating letter casing to convey sarcasm or mockery" '((emacs "27.1")) :commit "ff932d3d7cd65d958f98d3924e98bf727c45549b" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Altcaps Development" . "~protesilaos/altcaps@lists.sr.ht") :url "https://git.sr.ht/~protesilaos/altcaps")
